@@ -63,13 +63,14 @@ struct raop_callbacks_s {
     void  (*register_client) (void *cls, const char *device_id, const char *pk_str, const char *name);
     bool  (*check_register) (void *cls, const char *pk_str);
     void  (*export_dacp) (void *cls, const char *active_remote, const char *dacp_id);
+    void  (*video_reset) (void *cls);
 };
 typedef struct raop_callbacks_s raop_callbacks_t;
-raop_ntp_t *raop_ntp_init(logger_t *logger, raop_callbacks_t *callbacks, const char *remote, int remote_addr_len,
-                          unsigned short timing_rport, timing_protocol_t *time_protocol);
+raop_ntp_t *raop_ntp_init(logger_t *logger, raop_callbacks_t *callbacks, const char *remote,
+                          int remote_addr_len, unsigned short timing_rport, timing_protocol_t *time_protocol);
 
 RAOP_API raop_t *raop_init(raop_callbacks_t *callbacks);
-RAOP_API int raop_init2(raop_t *raop, int max_clients, const char *device_id, const char *keyfile);
+RAOP_API int raop_init2(raop_t *raop, int nohold, const char *device_id, const char *keyfile);
 RAOP_API void raop_set_log_level(raop_t *raop, int level);
 RAOP_API void raop_set_log_callback(raop_t *raop, raop_log_callback_t callback, void *cls);
 RAOP_API int raop_set_plist(raop_t *raop, const char *plist_item, const int value);
